
#ifndef __CLIENT_NET_INNER_TYPES_H_
#define __CLIENT_NET_INNER_TYPES_H_

#include <csignal>
#include <cstring>
#include <vector>
#include <memory>
#include <string>
#include <mutex>

#if (defined(WIN32) || defined(WIN64))
#else
#include <arpa/inet.h>
#endif

#include "json-c/json.h"
#include "Types.h"
#include "Noncopyable.h"

#if (!((defined(WIN32) || defined(WIN64))))
    constexpr CORE_HANDLE INVALID_CORE_HANDLE {static_cast<CORE_HANDLE>(0)};
#else
    #define INVALID_CORE_HANDLE (0)
#endif

class Object
{
};

class ObjectB : public Object
{
public:
    virtual void Die(void) = 0;
    virtual void SetHandle(CORE_HANDLE h) = 0;
    virtual CORE_HANDLE GetHandle(void) const = 0;
    virtual CORE_HANDLE GetBaseHandle(void) const = 0;
    virtual bool IsAnchor(void) const = 0;
    // 这里最好别有成员变量，否则android似乎容易出问题
};

struct BaseInfo
{
    std::string m_uid;
    std::string m_appName;
    std::string m_sn;
    std::string m_plat;
    std::string m_ver;
};

struct ScheduleInfos
{
    std::string m_protoFromServer;

    std::string m_mainAddr;
    std::vector<std::string> m_backAddr;

    std::string m_appName;
    std::string m_sn;
    std::string m_authKey;
    std::string m_clusterID;
    std::string m_streamKey;
    std::string m_uploadKey;
    std::string m_sn_alias;
    std::string m_publicMainAddr;
    std::string m_streamType {"all"};
    int m_version {0};
};

class JSONObjectGuard : private noncopyable
{
public:
    explicit JSONObjectGuard(struct json_object *obj) : m_obj(obj) {}
    JSONObjectGuard(void) = default;
    ~JSONObjectGuard()
    {
        release();
    }
    JSONObjectGuard &operator=(struct json_object *obj)
    {
        m_obj = obj;
        return *this;
    }

    json_object *Get()
    {
        return m_obj;
    }
    struct json_object *Release()
    {
        struct json_object *obj(m_obj);
        m_obj = nullptr;
        return obj;
    }
    void Reset(struct json_object *obj)
    {
        if (obj == m_obj) {
            return;
        }
        release();
        m_obj = obj;
    }
    operator bool() const
    {
        return (nullptr != m_obj);
    }

private:
    void release()
    {
        if (nullptr != m_obj) {
            (void)json_object_put(m_obj);
            m_obj = nullptr;
        }
    }

private:
    struct json_object *m_obj {nullptr};
};

typedef void (*event_inner_callback_t)(int c, enum EEvent e, const void *param, std::weak_ptr<Object> context);
typedef void (*rtmp_callback_t)(enum EEvent e, const void *param, std::weak_ptr<Object> context);
typedef void (*FBCallback)(CORE_HANDLE ctx, bool succeed);

enum class IDENTITY : short
{
    ID_UNKNOWN = -1, 
    ID_PUB, 
    ID_SUB
};

enum class ETransProto : short
{
    PROTOCOL_UNKNOWN = -1,
    PROTOCOL_TCP = 0,      // Only use TCP
    PROTOCOL_UDP = 1,      // Use UDP first, if failed then try TCP
    PROTOCOL_RTMP = 2,     // Only use RTMP
};

struct H264_context_t : public Object
{
	//yanjia
    H264_context_t(const std::weak_ptr<Object> &ctx, bool isTheKey,bool isHevc = false,int offset = 0) : context(ctx), is_key(isTheKey),is_hevc(isHevc),m_offset(offset) {}

    std::weak_ptr<Object> context;
    bool is_key;
    bool is_hevc;//yanjia for hevc
    int m_offset;//yanjia for hevc
}; 

struct AudioContext : public Object
{
    AudioContext(const std::weak_ptr<Object> &ctx, bool isTheAAC) : context(ctx), isAAC(isTheAAC) {}

    std::weak_ptr<Object> context;
    bool isAAC;
};

struct PersistentData
{
    std::string m_appName;
    int m_encodeType {-1};
    std::string m_encodeContext;
};

static void sigexit(int)
{
    return;
}

static inline void sigset()
{
#if (!defined(WIN32) && !defined(WIN64))
    struct sigaction actions;
    (void)memset(&actions, 0, sizeof(actions));
    (void)sigemptyset(&actions.sa_mask);
    actions.sa_flags = 0;
    actions.sa_handler = sigexit;
    (void)sigaction(SIGTERM, &actions, nullptr);

    actions.sa_handler = SIG_IGN;
    (void)sigaction(SIGPIPE, &actions, nullptr);
#else
    signal(SIGTERM, sigexit);
#endif
}

#endif

